/* dialogAPI.c -- dialog popups for x application */

/* mango/tango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include "dialogAPI.h"
#include "xgoCB.h"

extern Widget widget_to_shell(Widget w);

struct _dialogType {
   int type, isModal;
   Widget shell, pane, form, label, text;
   Widget buttonForm, lastButton;
};

dialogType *dialogCreate(Widget parent, int type, char *title, int isModal)
{
   dialogType *ret = NULL;
   
   if (parent)
   {
      char *theTitle = (title? title: "Input");

      ret = NEW(dialogType);
      if (ret)
      {
	 Widget panedChild = NULL;
	 
	 ret->type = type;
	 ret->isModal = isModal;
	 ret->shell =
	    XtVaCreatePopupShell("dialogShell",
				 topLevelShellWidgetClass, parent,
				 TITLE_RESOURCE, theTitle,
				 ICON_NAME_RESOURCE, theTitle,
				 NULL);
	 if (ret->shell)
	 {
#if defined(USE_MOTIF)
	    if (ret->type == DIALOG_FILENAME)
	    {
	       ret->form =
		  XtVaCreateManagedWidget( "fileSelectionBox",
					  xmFileSelectionBoxWidgetClass,
					  ret->shell,
					  NULL);
	       if (ret->form)
	       {
		  XtUnmanageChild(XmFileSelectionBoxGetChild(ret->form,
						 XmDIALOG_HELP_BUTTON));
	       }
	    }
	    else
	    {
	       ret->pane =
		  XtVaCreateManagedWidget( "panedChild",
					  xmPanedWindowWidgetClass, ret->shell,
					  NULL);
	       ret->form =
		  XtVaCreateManagedWidget( "formChild",
					  xmFormWidgetClass, ret->pane,
					  NULL);
	    }
#else
	    ret->pane =
	       XtVaCreateManagedWidget( "panedChild",
				       panedWidgetClass, ret->shell,
				       NULL);
	    ret->form =
	       XtVaCreateManagedWidget( "formChild",
				       formWidgetClass, ret->pane,
				       NULL);
#endif
	    XtAddCallback(ret->shell, POPDOWN_CALLBACK,
			  (XtCallbackProc)XtDestroyWidget_CB, ret->shell);
	    XtAddCallback(ret->shell, DESTROY_CALLBACK,
			  (XtCallbackProc)XtFree_CB, ret);
	 }
      }
   }
   return ret;
}

Widget dialogSetLabel(dialogType *dialog, char *labelString)
{
   Widget label = NULL;
   
   if (dialog && dialog->form)
   {
      if (labelString)
      {
	 if (!dialog->label)
	 {
#if defined(USE_MOTIF)
	    if (dialog->type == DIALOG_FILENAME)
	    {
	       dialog->label =
		  XmFileSelectionBoxGetChild(dialog->form,
					     XmDIALOG_SELECTION_LABEL);
	       XtVaSetValues(dialog->label,
			     STRING_ARG(XmNlabelString, labelString),
			     NULL);
	    }
	    else
	    {
	       dialog->label =
		  XtVaCreateManagedWidget( "dialogLabel",
					  xmLabelWidgetClass, dialog->form,
					  STRING_ARG(XmNlabelString,
						     labelString),
					  XmNtopAttachment, XmATTACH_FORM,
					  XmNleftAttachment, XmATTACH_FORM,
					  XmNrightAttachment, XmATTACH_FORM,
					  NULL);
	    }
#else
	    dialog->label =
	       XtVaCreateManagedWidget( "dialogLabel",
				       labelWidgetClass, dialog->form,
				       XtNlabel, labelString,
				       XtNborderWidth, 0,
				       NULL);
#endif
	 }
	 else
	 {
	    XtVaSetValues(dialog->label,
#if defined(USE_MOTIF)
			  STRING_ARG(XmNlabelString, labelString),
#else
			  XtNlabel, labelString,
#endif
			  NULL);
	 }
      }
      else /* label string is NULL */
      {
	 if (dialog->label)
	 {
	    XtDestroyWidget(dialog->label);
	    dialog->label = NULL;
	 }
      }
      label = dialog->label;
   }
   return label;
}

Widget dialogAddText(dialogType *dialog)
{
   Widget ret = NULL;
   
   if (dialog && dialog->pane && dialog->form)
   {
      if (!dialog->text)
      {
	 char *widgetName =
	    (dialog->type == DIALOG_FILENAME? "dialogFilename": "dialogText");
#if defined(USE_MOTIF)
	 dialog->text =
	    XtVaCreateManagedWidget( widgetName,
				    xmTextFieldWidgetClass, dialog->form,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_FORM,
				    NULL);
#else
	 dialog->text =
	    XtVaCreateManagedWidget(widgetName,
				    asciiTextWidgetClass, dialog->form,
				    XtNeditType, XawtextEdit,
				    XtNleft, XtChainLeft,
				    XtNright, XtChainRight,
				    NULL);
#endif
	 XtAddCallback(dialog->shell, POPUP_CALLBACK,
		       (XtCallbackProc)setWidthToParent_CB, dialog->text);
	 if (dialog->label)
	 {
	    XtVaSetValues(dialog->text,
#if defined(USE_MOTIF)
			  XmNtopAttachment, XmATTACH_WIDGET,
			  XmNtopWidget, dialog->label,
#else
			  XtNfromVert, dialog->label,
#endif
			  NULL);
	 }
      }
      ret = dialog->text;
   }
   return ret;
}

char *dialogGetText(dialogType *dialog)
{
   char *str = NULL, *temp;
   Widget text = dialog->text;
   
#if defined(USE_MOTIF)
   if (dialog->type == DIALOG_FILENAME || text == NULL)
      text = XmFileSelectionBoxGetChild(dialog->form,
					XmDIALOG_TEXT);
   if (XmIsText(text))
      str = XmTextGetString(text);
   else if (XmIsTextField(text))
      str = XmTextFieldGetString(text);
#else
   XtVaGetValues(text, XtNstring, &temp, NULL);
   str = XtNewString(temp);
#endif

   return str;
}

Widget dialogAddButton(dialogType *dialog, int buttonType, char *buttonLabel,
		       XtCallbackProc proc, XtPointer data, Boolean popdown)
{
   Widget ret = NULL;
   
   if (dialog && dialog->form)
   {
      if (buttonLabel == NULL)
      {
	 switch (buttonType)
	 {
	  case BUTTON_OK:	buttonLabel = "Ok";	break;
	  case BUTTON_APPLY: 	buttonLabel = "Apply";	break;
	  case BUTTON_CANCEL:	buttonLabel = "Cancel";	break;
	  case BUTTON_HELP:	buttonLabel = "Help";	break;
	  default:		buttonLabel = "Other";	break;
	 }
      }
      
#if defined(USE_MOTIF)
      if (dialog->type == DIALOG_FILENAME)
      {
	 char *callback = NULL;
	 
	 switch (buttonType)
	 {
	  case BUTTON_OK:
	    callback = XmNokCallback;
	    break;
	  case BUTTON_APPLY:
	    callback = XmNapplyCallback;
	    break;
	  case BUTTON_CANCEL:
	    callback = XmNcancelCallback;
	    break;
	  case BUTTON_HELP:
	    callback = XmNhelpCallback;
	    XtManageChild(XmFileSelectionBoxGetChild(dialog->form,
					      XmDIALOG_HELP_BUTTON));
	    break;
	 }

	 if (proc)
	    XtAddCallback(dialog->form, callback, proc, data);
	 if (popdown)
	    XtAddCallback(dialog->form, callback,
			  (XtCallbackProc)XtPopdown_CB, dialog->shell);
      }
      else
#endif
	 if (!dialog->buttonForm)
      {
#if defined(USE_MOTIF)
	 dialog->buttonForm =
	    XtVaCreateManagedWidget( "dialogButtonForm",
				    xmFormWidgetClass, dialog->pane,
				    NULL);
#else
	 dialog->buttonForm =
	    XtVaCreateManagedWidget("dialogButtonForm",
				    formWidgetClass, dialog->pane,
				    NULL);
#endif
      }
      if (dialog->buttonForm)
      {
#if defined(USE_MOTIF)
	 ret = 
	    XtVaCreateManagedWidget( "dialogButton",
				    xmPushButtonWidgetClass,
				    dialog->buttonForm,
				    STRING_ARG(XmNlabelString, buttonLabel),
				    NULL);
#else
	 ret =
	    XtVaCreateManagedWidget("dialogButton",
				    commandWidgetClass, dialog->buttonForm,
				    XtNfromHoriz, dialog->lastButton,
				    XtNlabel, buttonLabel,
				    NULL);
	 
#endif
	 if (ret)
	 {
	    if (proc)
	       XtAddCallback(ret, ACTIVATE_CALLBACK, proc, data);
	    if (popdown)
	       XtAddCallback(ret, ACTIVATE_CALLBACK,
			     (XtCallbackProc)XtPopdown_CB, dialog->shell);
	    
#if defined(USE_MOTIF)
	    if (dialog->lastButton)
	    {
	       XtVaSetValues(ret,
			     XmNleftAttachment, XmATTACH_WIDGET,
			     XmNleftWidget, dialog->lastButton,
			     NULL);
	    }
#endif
	 }
	 dialog->lastButton = ret;
      }
   }
   return ret;
}

int dialogPopup(dialogType *dialog)
{
   if (dialog && dialog->shell)
   {
      XtGrabKind grab = (dialog->isModal? XtGrabNonexclusive: XtGrabNone);
      
      XtPopup(dialog->shell, grab);
   }
}


int dialogErrorPopup(Widget parent, char *title, char *message, int isModal)
{
   dialogType *dialog =
      dialogCreate(widget_to_shell(parent), DIALOG_ERROR, title, isModal);
   if (dialog)
   {
      dialogSetLabel(dialog,  message);
      dialogAddButton(dialog, BUTTON_OK, NULL, NULL, NULL, True);
      dialogPopup(dialog);
   }
}

