/* goGame.h -- header for defining structures used in go */
   
/* mango/tango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#if !defined(GO_GAME_H)
#define GO_GAME_H

#define COMMENT_LEN	80
#define BOARD_MAX_LEN	400

#define BOARD_EMPTY	'.'
#define BOARD_STAR	'+'
#define BOARD_WHITE	'o'
#define BOARD_BLACK	'*'

#define PLAYER_NONE	0
#define PLAYER_BLACK 	1
#define PLAYER_WHITE	2
#define PLAYER_OBSERVER	3

#define MESSAGE_UNKNOWN		0
#define MESSAGE_COMMENT		1
#define MESSAGE_PLACE_STONE	2
#define MESSAGE_CAPTURE_STONE	3
#define MESSAGE_CAPTURE_GROUP	4
#define MESSAGE_UNDO_MOVE	5
#define MESSAGE_NOTIFY_SERVER	6
#define MESSAGE_NOTIFY_CLIENT	7
#define MESSAGE_NEW_GAME	8
#define MESSAGE_TRANSFER_HIST	9
#define MESSAGE_TRANSFER_DONE	10
#define MESSAGE_REQUEST_MOVE	11
#define MESSAGE_REQUEST_UNDO	12
#define MESSAGE_PASS		13
#define MESSAGE_DISCONNECT	14
#define MESSAGE_TERRITORY_ON	100
#define MESSAGE_TERRITORY_OFF	101

#define REASON_VALID		0
#define REASON_UNKNOWN		1
#define REASON_OBSERVER		2
#define REASON_NO_PLAYER	3
#define REASON_SUICIDE		4
#define REASON_NORMAL_KO	5
#define REASON_SUPER_KO		6
#define REASON_GAME_OVER	7

#define OPTION_BEEP_ON_MOVE	(1L<<0)
#define OPTION_GAME_DIRTY	(1L<<6)
#define OPTION_SUPER_KO_OFF	(1L<<7)

#define PLAY_HANDICAP	0
#define PLAY_FIRSTMOVE	1
#define PLAY_NORMAL	2
#define PLAY_PASSES	3

#if !defined(_XtIntrinsic_h)
typedef char Boolean;
#endif
#if !defined(True)
#define True 1
#endif
#if !defined(False)
#define False 0
#endif

#undef GO_GAME_DEFINED

typedef struct _goHistory goHistory;
typedef struct _goGame goGame;
typedef struct _goMessage goMessage;

extern char *playerNames[];

#define GO_GAME_DEFINED

int boardGetSpot(goGame *theGame, int i, int j);
int boardSetSpot(goGame *theGame, int i, int j, int val);
int isStarPoint(goGame *theGame, int i, int j);
void outputBoardText(FILE *out, char *board, int rows, int columns);

goGame *goGameCreate(int rows, int columns, int starPoint, int turn);
goGame *goGameCopy(goGame *orig, int keepHistory);
int goGameCompare(goGame *l, goGame *r);
int goGameSetId(goGame *theGame, int myId);
int goGameGetId(goGame *theGame);
int goGameGetKeepHistory(goGame *theGame);
int goGameSetKeepHistory(goGame *theGame, int historyOn);
int goGameGetNumPasses(goGame *theGame);
goGame *goGameClear(goGame *theGame);
goGame *goGameCleanHistory(goGame *theGame);
char *goGameBoardString(goGame *theGame);
goGame *goGameDestroy(goGame *theGame);
int goGameSetOption(goGame *theGame, long int option, int high);
int goGameGetOption(goGame *theGame, long int option);
int goGameSetPlayer(goGame *theGame, int player);
int goGameGetPlayer(goGame *theGame);
int goGameGetTurn(goGame *theGame);
int goGameSetTurn(goGame *theGame, int player);
int goGameSave(goGame *theGame, char *saveFile);
int goGameLoad(goGame *theGame, char *loadFile);

goHistory *goHistoryDestroy(goHistory *bye);
goGame *goGameAddToHistory(goGame *theGame, goMessage *message);
goMessage *goGameGetMessage(goGame *game);
goHistory *goHistorySearchLast(goHistory *from, int messageType);
goGame *goGameRemoveFromHistory(goGame *theGame, goMessage *message);

goMessage *goMessageCreate(void);
void goMessageSave(FILE *out, goMessage *mess);
goMessage *goMessageDestroy(goMessage *mess);
goMessage *goMessageCopy(goMessage *orig);
char *goMessageStr(goMessage *mess);
goMessage *goMessageFill(goMessage *fillMe, goGame *theGame,
			 int messageType, int x, int y, int player,
			 char *comment);
int goMessageCompare(goMessage *m1, goMessage *m2);
char *goMessageReasonStr(int reason);
int goMessageValid(goGame *theGame, goMessage *message, int *reason);
void recursiveRemoveStone(goGame *theGame, int boardx, int boardy,
			  int removeColor);
int undoLastMove(goGame *theGame, int *opponent);
void goMessageSend(goGame *theGame, int reason,
		   int x, int y, int player, char *text);
void goMessageApply(goGame *theGame, goMessage *message);

#endif

