/* socklink.c -- provides socket linkage functions between two processes */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include "socklinkP.h"
#ifdef __MINT__
#include "errno.foo"
#endif

#define BUF_SIZE 512
#define MYHOSTNAMELEN 255

qHead *qHeadCreate(void)
{
   return (qHead *)calloc(1, sizeof(qHead));
}

qHead *qHeadDestroy(qHead *head)
{
   qHead *ret = NULL;
   if (head) free(head);
   return ret;
}

int qHeadSocketFileDescr(qHead *head)
{
   int ret = -1;
   if (head) ret = head->sockfd;
   return ret;
}

qItem *qItemCreate(void)
{
   return (qItem *)calloc(1, sizeof(qItem));
}

qItem *qItemDestroy(qItem *item)
{
   qItem *ret = NULL;
   if (item) free(item);
   return ret;
}

qItem *enqItem(qHead *head, void *item, int itemSize)
{
   qItem *ret = NULL;
   
   if (head)
   {
      qItem *new = qItemCreate();

      if (new)
      {
	 new->info = item;
	 new->infoSize = itemSize;
	 strcpy(new->senderHost, "");
	 new->senderPort = -1;
	 if (head->tail)
	 {
	    head->tail->next = new;
	    head->tail = new;
	 }
	 else
	 {
	    head->head = new;
	    head->tail = new;
	 }
	 new->next = NULL;
	 ret = new;
      }
   }
   return ret;
}

void *deqItem(qHead *head, char *remoteHost, int *remotePort)
{
   void *ret = NULL;
   
   if (head)
   {
      qItem *item = head->head;

      if (item)
      {
	 if (item->next)
	 {
	    head->head = item->next;
	 }
	 else
	 {
	    head->head = NULL;
	    head->tail = NULL;
	 }
	 ret = item->info;
	 if (remoteHost)
	    strcpy(remoteHost, item->senderHost);
	 if (remotePort)
	    *remotePort = item->senderPort;
	 qItemDestroy(item);
      }
   }
   return ret;
}

int emptyHead(qHead *head)
{
   int ret = True;

   if (head && head->head && head->tail)
      ret = False;
   
   return ret;
}

/* return the open socket number if successful */

int serverConnect(int port, struct sockaddr_in *in_name, int type)
{
   int ld, ret = -1;
   
   ld = socket(AF_INET, type, 0);
   if(ld < 0)
   {
      perror("socket() failed");
      exit(1);
   }
   
   if (in_name)
   {
      in_name->sin_family = AF_INET;
      in_name->sin_port = htons(port);
      in_name->sin_addr.s_addr = htonl(INADDR_ANY);
      printf("Server waiting at port: %d...\n", port);
   
      if(bind(ld, (struct sockaddr *)in_name,
	      sizeof(struct sockaddr_in)) < 0)
	 perror("bind() failed"); 

      ret = ld;
      if (type == SOCK_STREAM)
      {
	 listen(ld, 5);
      }
   }
   return ret;
}


int clientConnect(char *host, int port, int localPort,
		  struct sockaddr_in *in_name,
		  struct sockaddr_in *to, int type)
{
   int sd, ret = -1;
   struct hostent *hptr;
   
   sd = socket(AF_INET, type, 0);
   if(sd < 0)
   {
      perror("socket() failed");
      exit(1);
   }
   
   if (in_name && to)
   {
      in_name->sin_family = AF_INET;
      in_name->sin_port = htons(localPort);
      in_name->sin_addr.s_addr = INADDR_ANY;

      if (type == SOCK_DGRAM)
      {
	 if(bind(sd, (struct sockaddr *)in_name,
		 sizeof(struct sockaddr_in)) < 0)
	    perror("bind() failed");
      }
   
      to->sin_family = AF_INET;
      to->sin_port = htons(port);
      hptr = gethostbyname(host);
      bcopy(hptr->h_addr, &to->sin_addr.s_addr, hptr->h_length);

      ret = sd;

      if (type == SOCK_STREAM)
      {
	 if (connect(sd, (struct sockaddr *)to, sizeof(struct sockaddr_in))
	     < 0)
	 {
	    perror("connect() failed");
	    ret = -1;
	 }
      }
   }

   return ret;
}

/*  Enable or disable non-blocking I/O mode. */
int setNonBlocking(int fd, int flag)
{
   int mode;

   if ((mode = fcntl(fd, F_GETFL, 0)) < 0)
      return -1;
#if defined(O_NONBLOCK)
   mode |= O_NONBLOCK;
#endif
#if defined(O_NDELAY)
   mode |= O_NDELAY;
#endif
#if defined(FNDELAY)
   mode |= FNDELAY;
#endif
   return fcntl(fd, F_SETFL, mode);
}


qHead *prepareSocket(int isServer, char *host, int port, int localPort,
		     int nonBlock, int socketType)
{
   qHead *ret = NULL;
   int sockType = (socketType == SOCK_TYPE_UDP? SOCK_DGRAM: SOCK_STREAM);

   ret = qHeadCreate();
   if (ret)
   {
      ret->port = port;
      ret->localPort = localPort;
      ret->type = sockType;
      if (isServer)
      {
	 ret->sockfd = serverConnect(ret->port, &(ret->here), ret->type);
	 if (ret->sockfd < 0) goto ErrorExit;
	 /* @@@@ could wait for the client here... */
	 ret->isServer = True;
	 ret->thereFilled = False;
	 if (ret->type == SOCK_STREAM)
	 {
	    int newsockfd;
	    int thereLen = sizeof(struct sockaddr_in);
	    
	    /* @@@ I think this will block -- and there is nothing that can
	     * be done about it */
	    newsockfd = accept(ret->sockfd,
			       (struct sockaddr *)&ret->there, &thereLen);
	    if (newsockfd >= 0)
	    {
	       struct hostent *hp;
	       char *hostname;
	       
	       /* stream server sockets have to fill in this information AFTER
		* having succesfully accept()'ed */
	       if((hp=gethostbyaddr((char *)&(ret->there.sin_addr),
				    sizeof(struct in_addr), AF_INET)) == NULL)
		  perror("unknown host calling");
	       if (hp)
		  hostname = hp->h_name;
	       else
		  hostname = inet_ntoa(ret->there.sin_addr);
	       strcpy(ret->remoteHost, hostname);
	       /* fill in remote information */
	       /*
		  bzero(head->there, sizeof(struct sockaddr_in));
		  head->there.sin_port = ptr->sin_port;
		  */
	       ret->there.sin_family = AF_INET;
	       bcopy(hp->h_addr, &ret->there.sin_addr.s_addr, hp->h_length);
	       ret->thereFilled = True;
	       
	       /* here's the important thing for stream sockets...
		* set the new socket descriptor to replace the old... */
	       close(ret->sockfd);
	       ret->sockfd = newsockfd;
	    }
	 }
      }
      else if (host)
      {
	 ret->sockfd = clientConnect(host, ret->port, ret->localPort,
				     &(ret->here), &(ret->there), ret->type);
	 if (ret->sockfd < 0) goto ErrorExit;
	 strcpy(ret->remoteHost, host);
	 ret->thereFilled = True;
	 ret->isServer = False;
      }
      /* make the socket be non-blocking */
      if (nonBlock)
	 setNonBlocking(ret->sockfd, True);
   }
   goto Exit;
 ErrorExit:
   ret = qHeadDestroy(ret);
 Exit:
   return ret;
}

qHead *pollSocket(qHead *head, int itemSize)
{
   char *buf = (char *)calloc(1, itemSize);
   
   if (head)
   {
      int thereLen = sizeof(struct sockaddr_in);
      int recvret = -1;
      
      bzero(buf, itemSize);
      errno = 0;


      if (head->type == SOCK_STREAM)
	 recvret = recv(head->sockfd, buf, itemSize, 0);
      else
	 recvret = recvfrom(head->sockfd, buf, itemSize, 0,
			    (struct sockaddr *)&head->there, &thereLen);
      if (recvret >= 0 && errno != EWOULDBLOCK)
      {
	 qItem *new = NULL;
	    
	 /* if there is something waiting on the socket... read it in */
	 if (recvret = itemSize)
	    new = enqItem(head, buf, itemSize);
	 else
	    printf("recv ret got something funky: %d\n", recvret);
	    
	 if (new && head->type == SOCK_DGRAM && !head->thereFilled)
	 {
	    struct hostent *hp;
	    char *hostname;

	    /* printf("deqItem: %x\n", new); */
	    /* dgram server sockets have to fill in this information AFTER
	     * having received the first sent message */
	    if((hp=gethostbyaddr((char *)&(head->there.sin_addr),
				 sizeof(struct in_addr), AF_INET)) == NULL)
	       perror("unknown host calling");
	    if (hp)
	       hostname = hp->h_name;
	    else
	       hostname = inet_ntoa(head->there.sin_addr);
	    strcpy(head->remoteHost, hostname);
	    /* fill in remote information */
	    /*
	    bzero(head->there, sizeof(struct sockaddr_in));
	    head->there.sin_port = ptr->sin_port;
	    */
	    head->there.sin_family = AF_INET;
	    bcopy(hp->h_addr, &head->there.sin_addr.s_addr, hp->h_length);
	    head->thereFilled = True;
	 }
	 if (new)
	 {
	    /* @@@@ every time? */
	    strcpy(new->senderHost,head->remoteHost);
	    new->senderPort = htons(head->there.sin_port);
	 }
	 else free(buf);
      }
      else if (recvret < 0 && errno == 0)
      {
	 perror("recv()/recvfrom() failed");
	 free(buf);
      }
   }
   return head;
}

int recvSocket(qHead *head, void *buffer, int itemSize)
{
   int recvret = -1;
   if (head && buffer && itemSize)
   {
      int thereLen = sizeof(struct sockaddr_in);
      
      bzero(buffer, itemSize);
      errno = 0;

      if (head->type == SOCK_STREAM)
	 recvret = recv(head->sockfd, buffer, itemSize, 0);
      else
	 recvret = recvfrom(head->sockfd, buffer, itemSize, 0,
			    (struct sockaddr *)&head->there, &thereLen);
      if (recvret >= 0 && errno != EWOULDBLOCK)
      {
	 if (recvret != itemSize)
	    printf("recv ret got something funky: %d\n", recvret);
	    
	 if (head->type == SOCK_DGRAM && !head->thereFilled)
	 {
	    struct hostent *hp;
	    char *hostname;

	    /* printf("deqItem: %x\n", new); */
	    /* dgram server sockets have to fill in this information AFTER
	     * having received the first sent message */
	    if((hp=gethostbyaddr((char *)&(head->there.sin_addr),
				 sizeof(struct in_addr), AF_INET)) == NULL)
	       perror("unknown host calling");
	    if (hp)
	       hostname = hp->h_name;
	    else
	       hostname = inet_ntoa(head->there.sin_addr);
	    strcpy(head->remoteHost, hostname);
	    /* fill in remote information */
	    /*
	    bzero(head->there, sizeof(struct sockaddr_in));
	    head->there.sin_port = ptr->sin_port;
	    */
	    head->there.sin_family = AF_INET;
	    bcopy(hp->h_addr, &head->there.sin_addr.s_addr, hp->h_length);
	    head->thereFilled = True;
	 }
	 /* @@@@ every time? */
	 /*
	 strcpy(new->senderHost,head->remoteHost);
	 new->senderPort = htons(head->there.sin_port);
	 */
      }
      else if (recvret < 0)
      {
	 perror("recv()/recvfrom() failed");
      }
   }
   return recvret;
}

qHead *sendSocket(qHead *head, void *item, int itemSize)
{
   if (head)
   {
      if (head->thereFilled)
      {
	 int sendret = -1, tries = 7, sendAgain = True, time = 1;

	 while (sendAgain)
	 {
	    if (head->type == SOCK_STREAM)
	       sendret = send(head->sockfd, (char *)item, itemSize, 0);
	    else
	       sendret = sendto(head->sockfd, (char *)item, itemSize, 0,
				(struct sockaddr *)&head->there,
				sizeof(struct sockaddr_in));
	    /*  printf("head, socket, sendto ret, errno: %x, %d, %d, %d\n",
	     *         head, head->sockfd, sendret, errno);
	     */
	    tries--;
	    if (sendret < 0 && tries &&
		(errno == EAGAIN ||
		 errno == EWOULDBLOCK ||
		 errno == EDEADLK))
	    {
	       printf("<Yawn>... sleeping.  Will try again in a sec.\n");
	       sleep(time);
	       time *= 2;
	    }
	    else
	       sendAgain = False;
	 }
	 if (sendret < 0)
	 {
	    perror("sendto failed");
	    fprintf(stderr, "errno: %d\n", errno);
	 }
      }
      else
	 printf("Cannot send package to unknown client.\n");
   }
   return head;
}

qHead *closeSocket(qHead *head)
{
   if (head)
   {
      /* postpone destroying the actual queue items */
      close(head->sockfd);
      head = qHeadDestroy(head);
   }
   return head;
}

char *getThisHostName(int domain)
{
   char hostname[MYHOSTNAMELEN+1];

   gethostname(hostname, sizeof (hostname));
   if (domain)
   {
      char domainname[MYHOSTNAMELEN+1];
      int success = 0;
      
      strcpy(domainname, "");
#ifndef __MINT__
      /* @@@@ this function doesn't seem to work (returns empty string) */
      success = getdomainname(domainname, sizeof (domainname));
      if (success != 0)
	 perror("getdomainname()");
      else if (strlen(domainname) == 0)
#endif
      {
	 /* the function wasn't successful... get our own domain name */
	 FILE *resolv;

	 if (resolv = fopen("/etc/resolv.conf", "r"))
	 {
	    char line[MYHOSTNAMELEN+1];
	    int found = False;

	    while (!found && fgets(line, MYHOSTNAMELEN, resolv))
	    {
	       if (strstr(line, "domain"))
	       {
		  sscanf(line, "%*s %s", domainname);
		  found = True;
	       }
	    }
	    fclose(resolv);
	 }
      }
     
      if (strlen(domainname))
      {
	 strcat(hostname, ".");
	 strcat(hostname, domainname);
      }
   }
   return hostname;
}


int fillInAddresses(char **hisName, char **hisHost,
		    char **myName, char **myHost)
{
   static char hHost[MYHOSTNAMELEN+1], mHost[MYHOSTNAMELEN+1], *at;
   static struct passwd *pwd;
   int ret = False, useDomain = False;

   if (hisHost && strEmpty(*hisHost))
   {
      strcpy(hHost, getThisHostName(False));
      *hisHost = hHost;
   }
   /* if domain is in his hostname, put my domain here as well */
   at = strchr(*hisHost, '.');
   useDomain = (at? True: False);
   strcpy(mHost, getThisHostName(useDomain));
   *myHost = mHost;
   *myName = getlogin();
   if (strEmpty(*myName))
   {
      /* try an alternate method */
      pwd = getpwuid(getuid());
      if (pwd)
	 *myName = pwd->pw_name;
   }
   if (strEmpty(*myName))
      fprintf(stderr,"Could not get login id information.\n");
   else if (strEmpty(*myHost))
      fprintf(stderr,"Could not get local hostname information.\n");
   else if (strEmpty(*hisHost))
      fprintf(stderr,"Could not get remote hostname information.\n");
   else
      ret = True;
   return ret;
}

#define WRITE_SUCCESS	0

int notifyHisTerminal(char *appName, char *method, 
		      char *hisName, char *hisHost,
		      char *myName, char *myHost, int port)
{
   char message[512], command[512], myAddr[128], hisAddr[128];
   int ret = False, wrret;

   if (strcmp(myHost,hisHost)==0)
   {
      if (myName) strcpy(myAddr, myName);
      else strcpy(myAddr, "unknown");
      strcpy(hisAddr, hisName);
   }
   else
   {
      sprintf(myAddr, "%s@%s", myName, myHost);
      sprintf(hisAddr, "%s@%s", hisName, hisHost);
   }
   
   /* HACK alert!  Ideally I should not use system() for this!
    * Got any better ideas?
    * (Ideally I would like to notify him through an inetd service
    * like talkd or ntalkd -- but all these services display their
    * won canned messages)
    */
   if (strstr(method, "none"))
      ret = True;
   else
   {
      sprintf(message, "%s wants to play a game of go with you.\n"
	      "To play go, type: %s -c %s -p %d",
	      myAddr, appName, myAddr, port);
      sprintf(command, "echo '%s' | %s %s", message, method, hisAddr);
      wrret = system(command);
      if (wrret == WRITE_SUCCESS)
	 ret = True;
   }
   return ret;
}

#if defined(SOCKLINK_MAIN)
int main(int argc, char *argv[])
{
   if(argc != 2 && argc != 4)
   {
      printf("Usage: %s <port> [<host> <localPort>]\n", argv[0]);
      exit(1);
   }
   else if (argc == 2)
   {
      /* be a server */
      serverConnect(atoi(argv[1]));
   }
   else
   {
      /* be a client */
      clientConnect(argv[2], argv[1], argv[3]);
   }
}
#endif
