/* strmisc.c -- miscellaneous string manipulation functions */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "strmisc.h"

#if !defined(max)
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

/*------------------------------------------------------------------------*/

/* given a string sets the width (in characters) and the height (in lines)
 * returns width + height (non-zero if string not empty)
 * width and height pointer can be NULL (those values are only set if
 * the pointers are non-NULL)
 */

int strGetGeometry(char *string, int *width, int *height)
{
   int w = 0, h = 0, workw = 0;
   char *parse;

   for (parse = string; parse && *parse; parse++)
      {
      if (*parse == '\n')
         {
         h++;
         w = max(w, workw);
         workw = 0;
         }
      else
         {
         workw++;
         }
      }
   w = max(w, workw);
   if (w) h++;
   if (width) *width = w;
   if (height) *height = h;
   return (w + h);
}

/*------------------------------------------------------------------------*/

/* given a string trims any spaces (all white space and EOF marks) off the
 * end of the string (actually modifies string memory),
 * returns the same char * (with spaces trimmed) */

char *strTrimEnd(char *str)
{
   char *retstr = str;

   if (str)
      {
      int end = strlen(retstr)-1;
      
      while (end >= 0 && ( isspace(retstr[end]) || (retstr[end] == 26) ) )
         {
         retstr[end] = '\0';
         end--;
         }
      }
   return retstr;
}

/*------------------------------------------------------------------------*/

/* given a string trims any spaces (all white space and EOF marks) off the
 * beginning of the string (actually modifies string memory),
 * returns the same string (with spaces trimmed) */

char *strTrimBeginning(char *str)
{
   char *retstr = str;

#if defined(MEMMOVE)
   if (str)
      {
      int nonSpace = 0;

      while (retstr[nonSpace] && ( isspace(retstr[nonSpace]) ||
             (retstr[nonSpace] == 26) ))
         nonSpace++;
      if (nonSpace)
         {
         char *src = &(retstr[nonSpace]);
         int len = strlen(src);
         memmove(retstr, src, len);
         retstr[len] = '\0';
         }
      }
#endif
   return retstr;
}

/*------------------------------------------------------------------------*/

/* given a string returns 1 if the char * is NULL, if its the empty string
 * (i.e. str[0] == '\0'), or if the string contains only white space
 * (see description for isspace)...
 * otherwise the string is not empty and 0 is returned */

int strEmpty(char *str)
{
   int retval = 0;

   if (str == NULL)
      retval = 1;
   else
      {
      int nonSpace = 0;

      while (str[nonSpace] &&
             ( isspace(str[nonSpace]) || (str[nonSpace] == 26) ))
         nonSpace++;
      if (str[nonSpace] == '\0')
         retval = 1;
      }
   return retval;
}

/*------------------------------------------------------------------------*/

#define LARGE_STRING_LEN 256

/* given a string and a matching string, removes any occurance of match
 * and replaces it with repl
 * returns the same (updated) string */

char *strReplace(char *str, char *match, char *repl)
{
   char tmp[LARGE_STRING_LEN], *use, *at;

   if (strlen(str) < LARGE_STRING_LEN)
      while (at = strstr(str, match))
         {
         strcpy(tmp,at);
         *at = '\0';
         strcat(str, repl);
         at = tmp + strlen(match);
         strcat(str, at);
         }
   return str;
}

/*------------------------------------------------------------------------*/

/* removes all non-printable ascii characters from a given string
 * returns the string */

char *strRemoveNonPrint(char *str)
{
#if defined(MEMMOVE)
   char *scan;
   int len;

   for (scan = str; scan && *scan; scan++)
      {
      if (!isprint(*scan))
         {
         len = strlen(scan+1);
         memmove(scan, scan+1, len+1);
         }
      }
#endif
   return str;
}

/*------------------------------------------------------------------------*/

/* pad the end of a string with spaces to fit length "length" */

void strPadEnd(char *str, int length)
{
   int j;
   
   for (j = strlen(str); j < length; j++)
      str[j] = ' ';

   str[j] = '\0';
}



