/* xIFace.h -- the go interface description header file */

#if !defined(GO_INTERFACE_H)
#define GO_INTERFACE_H

#include "goGame.h"
#include "goScore.h"
#include "socklink.h"

#define CONNECTION_SINGLE_INSTANCE 	0	/* when go started w/o -c */
#define CONNECTION_SHARED_FILE		1	/* not currently used */
#define CONNECTION_SOCKET		2	/* 2 player socket connect'n */

typedef struct _goGameInterface goGameInterface;
typedef struct _goInterface goInterface;

struct _goGameInterface {
   goInterface *iface;
   goGame *game;
   goTerritory *territory;
   int scoringRules;
   int suppressOutput;
   int connectionType;
   qHead *connection;
   XtInputId socketInput;
};

/* xIFace.c */

goGameInterface *goGameInterfaceCreate(int rows, int columns,
				       int starPoint, int turn, int rules);
goGameInterface *goGameInterfaceDestroy(goGameInterface *ggi);
Widget goGameInterfaceGetBoard(goGameInterface *ggi);
int goGameInterfaceSetOption(goGameInterface *ggi, long int option, int high);
int goGameInterfaceGetOption(goGameInterface *ggi, long int option);
int goGameInterfaceSetId(goGameInterface *ggi, int myId);
int goGameInterfaceDisconnect(goGameInterface *ggi);
void goInterfaceSetColors(goGameInterface *ggi,
			  unsigned long grid,
			  unsigned long shadow,
			  unsigned long bright,
			  unsigned long light,
			  unsigned long dull,
			  unsigned long player1,
			  unsigned long player2);
int goGameInterfaceConnectionType(goGameInterface *ggi);
int goGameInterfaceSetConnectionType(goGameInterface *ggi, int connectionType);
qHead *goGameInterfaceGetConnection(goGameInterface *ggi);
int goGameInterfaceSetConnection(goGameInterface *ggi, qHead *connection);
void showTurnLabels(goGameInterface *ggi);
void showInvalidReason(goGameInterface *ggi, int reason);
void undoMoveCB(Widget w, goGameInterface *ggi, XtPointer nothing);
void ggiMessageSend(goGameInterface *ggi, int maybeBeep, int reason,
		    int x, int y, int player, char *text);
void ggiMessageApply(goGameInterface *ggi, goMessage *message,
		     int beepOverride);
void xgoInputProc(goGameInterface *ggi, int *sourcePtr, XtInputId *id);

/* xIFaceBoard.c */

void beepGame(goGameInterface *ggi);
void removeGroupOfStonesCB(Widget w, goGameInterface *ggi, XtPointer nothing);
void inputBoardCB(Widget w, XtPointer info, XEvent *event, Boolean *dispatch);
void highlightLastStone(goGameInterface *ggi);
void drawStone(goGameInterface *ggi, int i, int j);
void drawIntersection(goGameInterface *ggi, int i, int j);
void getAspectInfo(goGameInterface *ggi);
void drawWholeBoard(goGameInterface *ggi);
void redrawBoardCB(Widget w, XtPointer info, XEvent *event, Boolean *dispatch);
void drawBoardDifferent(goGameInterface *ggi, goGame *orig);

#endif
