/* go.c -- main file */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

/* include files: */
#include "xgo.h"		/* all Xm, X11, and std includes */

#define MANGO_APP_RES
#include "xgoRes.h"

/* global variables: */

Widget goTopLevel;
XtAppContext goContext;
Display *goDisplay;
Screen *goScreen;
Widget mainWidget = NULL;
goAppData appDefaults;

/* global scoring variables (see goScore.c): */

extern int decayMultiple, influenceDistance, ownershipMult;
extern float ownershipFloor;
static char *progTitle = "Mango v1.0.1";

#define NEW_TOP_SHELL
#define INPUT_PROC

/* build the popupShell: return value of function */

Widget goBuild(Widget parent, goGameInterface *ggi, char *titleExtra)
{
   Widget topShell = NULL;
   char title[128];

   if (titleExtra)
      sprintf(title, "%s: %s", progTitle, titleExtra);
   else
      strcpy(title, progTitle);
#if defined(NEW_TOP_SHELL)
   topShell =
      XtVaCreatePopupShell("goShell",
			   topLevelShellWidgetClass, parent,
			   TITLE_RESOURCE, title,
			   ICON_NAME_RESOURCE, title,
			   NULL);
#else
   XtVaSetValues(parent,
		 TITLE_RESOURCE, title,
		 ICON_NAME_RESOURCE, title,
		 NULL);
   topShell = parent;
#endif

   if (topShell)
   {
      Widget formChild = NULL, panedChild = NULL;
      
#if defined(USE_MOTIF)
      formChild =
	XtVaCreateManagedWidget( "formChild",
				xmFormWidgetClass, topShell,
				NULL);
      if (formChild)
	 panedChild =
	    XtVaCreateManagedWidget( "panedChild",
				    xmPanedWindowWidgetClass, formChild,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_FORM,
				    XmNbottomAttachment, XmATTACH_FORM,
				    NULL);
#else
      panedChild =
	 XtVaCreateManagedWidget( "panedChild",
				 panedWidgetClass, topShell,
				 NULL);
#endif

      if (panedChild)
      {
	 Widget area;

	 /* build the areas of the panedChild */
#if defined(USE_MOTIF)
	 area = menuBuild(formChild, ggi);
	 if (area)
	    XtVaSetValues(panedChild,
			  XmNtopAttachment, XmATTACH_WIDGET,
			  XmNtopWidget, area,
			  NULL);
#else
	 area = menuBuild(panedChild, ggi);
#endif
	 if (goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
	 {
	    /* build a comment area for communication purposes */
	    area = commentBuild(panedChild, ggi);
	 }
	 area = statusBuild(panedChild, ggi);
	 area = boardAreaBuild(panedChild, ggi);
	 if (goGameInterfaceConnectionType(ggi) == CONNECTION_SINGLE_INSTANCE)
	 {
	    setPlayerBlackCB(goGameInterfaceGetBoard(ggi), ggi, NULL);
	 }
      }
   }
   XtAddCallback(topShell, POPDOWN_CALLBACK,
		 (XtCallbackProc)XtDestroyWidget_CB, topShell);
#if defined(NEW_TOP_SHELL)
   XtPopup(topShell, XtGrabNone);
#else
   XtRealizeWidget(topShell);
#endif
   return topShell;
}

/* default resources for the application: */

String fallbacks[] = {
   "XMango*fontList: 8x13",
   "XMango*background: skyblue",
   "XMango*foreground: black",
   "XMango*boardFrame.background: wheat",
   "XMango*boardFrame.shadowThickness: 5",
   "XMango*board.background: wheat",
   "XMango*board.foreground: black",
   "XMango*board.width: 350",
   "XMango*board.height: 350",
#if !defined(USE_MOTIF)
   "XMango*dialogText.Translations: #override <Key>Return: focus-in()",
   "XMango*dialogFilename.Translations: #override "
      "<Key>Return: focus-in()\\n"
      "<Key>space: focus-in()",
#endif
   NULL
   };

void parseArgInfo(int argc, char *argv[],
		  char **name, char **host, char **terminal,
		  int *server, int *port)
{
   int i;

   *name = NULL;
   *host = NULL;
   *terminal = NULL;
   if (server) *server = True;
   if (port) *port = 6000 + getpid()%1000;
   for (i = 0; i < argc; i++)
   {
      if (strcmp(argv[i],"-c")==0 && (i+1) < argc)
      {
	 char *at;
	 
	 *name = argv[i+1];
	 if (at = strchr(*name, '@'))
	 {
	    *host = (at+1);
	    *at = '\0';
	 }
      }
      if (strcmp(argv[i],"-p")==0 && server && port && (i+1)<argc)
      {
	 *server = False;
	 *port = atoi(argv[i+1]);
      }
   }
   
}

/* main function: */

int main(int argc, char *argv[])
{
   goGameInterface *ggi = NULL;
   char *hisName = NULL, *hisTerminal = NULL, *hisHost = NULL;
   char *myName = NULL, *myHost = NULL, *myAddr = NULL;
   int isServer, connectPort, localPort, gamePort;
   int connectionType, rules;
   qHead *connection;
   char *extra = NULL;

   parseArgInfo(argc, argv, &hisName, &hisHost, &hisTerminal,
		&isServer, &connectPort);
   localPort = 7000 + getpid()%1000;
   gamePort = localPort;
   connectionType = CONNECTION_SINGLE_INSTANCE;

   /* initialize Xt interface stuff */
   goTopLevel =
      XtAppInitialize(&goContext,
		      "XMango", NULL, 0, &argc, argv,
		      fallbacks, NULL, 0);
   goDisplay = XtDisplay(goTopLevel);
   goScreen = XDefaultScreenOfDisplay(goDisplay);

   /* check any application defaults */
   XtVaGetApplicationResources(goTopLevel, &appDefaults,
			       goResources, XtNumber(goResources),
			       NULL);

   /* set the scoring defaults */
   decayMultiple = appDefaults.decayMultiple;
   influenceDistance = appDefaults.influenceDistance;
   ownershipMult = (int)appDefaults.ownershipMult;
   ownershipFloor = appDefaults.ownershipFloor;
   if (strstr(appDefaults.scoringRules,"hin"))
      rules = RULE_TYPE_CHINA;
   else
      rules = RULE_TYPE_JAPAN;
   
   /* establish a connection to the other party */
   /* we may be a server... if so, write a message to the desired party */
   if (hisName)
   {
      fillInAddresses(&hisName, &hisHost, &myName, &myHost);

      /* connect to opponent through a socket */
      if (isServer)
      {
	 gamePort = connectPort;
	 if (!notifyHisTerminal(argv[0], appDefaults.inviteMethod,
				hisName, hisHost,
				myName, myHost, connectPort))
	 {
	    /* if remote user is not logged on (or some other problem) abort */
	    fprintf(stderr,"Unable to connect to opponent.\n");
	    exit_CB(NULL, NULL, NULL);
	 }
      }
      /* printf("myId: %d\n", gamePort); */
      /* 5th param == is non blocking? */
      connection =
	 prepareSocket(isServer, hisHost, connectPort, localPort,
		       False, SOCK_TYPE_TCP);
      if (connection)
	 connectionType = CONNECTION_SOCKET;
      else
	 fprintf(stderr,"Socket connection failed.  "
		 "Coming up in single user mode...\n");
   }

   /* set up the game information itself (True = keep history of game) */
   ggi = goGameInterfaceCreate(appDefaults.numColumns, appDefaults.numRows,
			       appDefaults.starPoint, PLAYER_BLACK, rules);
   if (ggi)
   {
      if (appDefaults.moveBeep)
	 goGameInterfaceSetOption(ggi, OPTION_BEEP_ON_MOVE, True);
      if (appDefaults.superKoOff)
	 goGameInterfaceSetOption(ggi, OPTION_SUPER_KO_OFF, True);
      goGameInterfaceSetId(ggi, gamePort);
      goGameInterfaceSetConnectionType(ggi, connectionType);
      goInterfaceSetColors(ggi,
			   (unsigned long)appDefaults.grid,
			   (unsigned long)appDefaults.shadow,
			   (unsigned long)appDefaults.bright,
			   (unsigned long)appDefaults.light,
			   (unsigned long)appDefaults.dull,
			   (unsigned long)appDefaults.player1,
			   (unsigned long)appDefaults.player2);
   }
   else
   {
      fprintf(stderr, "Unable to allocate game memory.\n");
      exit(1);
   }

   /* actually build the main widget panels */
   if (goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
   {
      extra = (isServer? "server": "client");
   }
   mainWidget = goBuild(goTopLevel, ggi, extra);

   XtAddCallback(mainWidget,DESTROY_CALLBACK,
		 (XtCallbackProc)goGameInterfaceDestroyCB, ggi);
   XtAddCallback(mainWidget,DESTROY_CALLBACK,
		 (XtCallbackProc)exit_CB, ggi);

   if (goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
   {
      goGameInterfaceSetConnection(ggi, connection);
      if (!isServer)
      {
	 ggiMessageSend(ggi, True, MESSAGE_NOTIFY_SERVER, 0, 0,
			PLAYER_NONE, myName);
      }

      ggi->socketInput =
	 XtAppAddInput(goContext, qHeadSocketFileDescr(connection),
		       (XtPointer)XtInputReadMask,
		       (XtInputCallbackProc)xgoInputProc, ggi);
   }

   XtAppMainLoop(goContext);
}

