/* xgoAthena.c -- Athena interface for xgo */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include "xgo.h"
#include "xIFaceP.h"
#include "dialogAPI.h"

Widget widget_to_shell(Widget w)
{
   /* @@@@ don't really care about menu shells... */
   while (w && !XtIsShell(w))
      w = XtParent(w);
      
   return(w);
}


Widget menuBuild(Widget parent, goGameInterface *ggi)
{
   Widget menu = NULL;
   Widget exitB, playerBB, playerWB, removeSB, undoB, passB;
   Widget clearB, saveB, loadB, scoreB;

   menu = XtVaCreateManagedWidget("menu", boxWidgetClass, parent, NULL);
   
   clearB = XtVaCreateManagedWidget("New", commandWidgetClass, menu, NULL);
   XtAddCallback(clearB, XtNcallback,
		 (XtCallbackProc)goGameInterfaceClearCB, ggi);
   
   saveB = XtVaCreateManagedWidget("Save", commandWidgetClass, menu, NULL);
   XtAddCallback(saveB, XtNcallback,
		 (XtCallbackProc)goGameInterfaceSaveCB, ggi);
   
   loadB = XtVaCreateManagedWidget("Load", commandWidgetClass, menu, NULL);
   XtAddCallback(loadB, XtNcallback,
		 (XtCallbackProc)goGameInterfaceLoadCB, ggi);
   
   scoreB = XtVaCreateManagedWidget("Score", commandWidgetClass, menu, NULL);
   XtAddCallback(scoreB, XtNcallback,
		 /* (XtCallbackProc)goGameInterfaceScoreCB */
		 (XtCallbackProc)ggiCalcTerritoryCB, ggi);
   
   playerBB = XtVaCreateManagedWidget("Set Black",
				  commandWidgetClass, menu,
				  NULL);
   XtAddCallback(playerBB, XtNcallback,
		 (XtCallbackProc)setPlayerBlackCB, ggi);
   
   playerWB = XtVaCreateManagedWidget("Set White",
				  commandWidgetClass, menu,
				  NULL);
   XtAddCallback(playerWB, XtNcallback,
		 (XtCallbackProc)setPlayerWhiteCB, ggi);

   undoB = XtVaCreateManagedWidget("Undo",
				   commandWidgetClass, menu,
				   NULL);
   XtAddCallback(undoB, XtNcallback,
		 (XtCallbackProc)undoMoveCB, ggi);
   
   passB = XtVaCreateManagedWidget("Pass",
				   commandWidgetClass, menu,
				   NULL);
   XtAddCallback(passB, XtNcallback,
		 (XtCallbackProc)playerPassCB, ggi);
   
   removeSB = XtVaCreateManagedWidget("Remove Group",
				  commandWidgetClass, menu,
				  NULL);
   XtAddCallback(removeSB, XtNcallback,
		 (XtCallbackProc)removeGroupOfStonesCB, ggi);
   XtSetSensitive(removeSB, False);
   ggi->iface->removeButton = removeSB;
   
   exitB = XtVaCreateManagedWidget("Exit",
				   commandWidgetClass, menu,
				   NULL);
   XtAddCallback(exitB, XtNcallback,
		 (XtCallbackProc)exit_CB, ggi);
  
   return menu;
}

Widget statusBuild(Widget parent, goGameInterface *ggi)
{
   Widget box = NULL;

   box = XtVaCreateManagedWidget("status",
				 boxWidgetClass, parent,
				 NULL);
   
   ggi->iface->showMe =
      XtVaCreateManagedWidget("showMe",
			      labelWidgetClass, box,
			      XtNlabel, "Select Color",
			      NULL);
   ggi->iface->showTurn =
      XtVaCreateManagedWidget("showTurn",
			      labelWidgetClass, box,
			      XtNlabel, "Select Color",
			      NULL);
   
   return box;
}

void setTurnLabel(goGameInterface *ggi)
{
   if (ggi && ggi->iface && ggi->iface->showTurn)
   {
      char info[100];

      sprintf(info, "%s to play", playerNames[goGameGetTurn(ggi->game)]);
      info[0] = toupper(info[0]);
      XtVaSetValues(ggi->iface->showTurn,
		    XtNlabel, info,
		    NULL);
   }
}

void setMeLabel(goGameInterface *ggi)
{
   if (ggi && ggi->iface && ggi->iface->showMe)
   {
      char info[100];

      sprintf(info, "You are %s", playerNames[goGameGetPlayer(ggi->game)]);
      XtVaSetValues(ggi->iface->showMe,
		    XtNlabel, info,
		    NULL);
   }
}

#define DRAWING_AREA_WIDGET_CLASS coreWidgetClass

Widget boardAreaBuild(Widget parent, goGameInterface *ggi)
{
   Widget board = NULL, frame = NULL;

   frame = parent;

   if (frame)
      board = XtVaCreateManagedWidget("board",
				      DRAWING_AREA_WIDGET_CLASS, frame,
				      NULL);
   if (board && ggi && ggi->iface)
   {
     int i;
     
     ggi->iface->area = board;
     XtAddEventHandler(board, ExposureMask, False,
		       redrawBoardCB, (XtPointer)ggi);
     XtAddEventHandler(board, StructureNotifyMask, False,
		       redrawBoardCB, (XtPointer)ggi); /* handles resize */
     XtAddEventHandler(board, ButtonPressMask, False,
		       inputBoardCB, (XtPointer)ggi);
   }
   
   return board;
}

#define COMMENT_ROWS	3

Widget commentBuild(Widget parent, goGameInterface *ggi)
{
   Widget commentArea = NULL;
   
   if (parent && ggi)
   {
      Widget viewport;
      Dimension height;
      
      commentArea = XtVaCreateManagedWidget("commentArea",
					    formWidgetClass, parent,
					    NULL);
 
      ggi->iface->textArea =
	 XtVaCreateManagedWidget("textArea",
				 asciiTextWidgetClass, commentArea,
				 XtNscrollHorizontal, XawtextScrollWhenNeeded,
				 XtNscrollVertical, XawtextScrollWhenNeeded,
				 XtNtop, XtChainTop,
				 XtNbottom, XtChainBottom,
				 XtNleft, XtChainLeft,
				 XtNright, XtChainRight,
				 XtNeditType, XawtextRead,
				 NULL);
      height = 0;
      XtVaGetValues(ggi->iface->textArea, XtNheight, &height, NULL);
      if (height)
      {
	 height *= COMMENT_ROWS;
	 
	 XtVaSetValues(ggi->iface->textArea, XtNheight, height, NULL);
      }
      
      ggi->iface->textEntry =
	 XtVaCreateManagedWidget("textEntry",
				 asciiTextWidgetClass, commentArea,
				 XtNeditType, XawtextEdit,
				 XtNfromVert, ggi->iface->textArea,
				 XtNright, XtChainRight,
				 XtNleft, XtChainLeft,
				 XtNtop, XtChainBottom,
				 XtNbottom, XtChainBottom,
				 NULL);
      if (ggi->iface->textEntry)
      {
	 Widget source;
	 
	 XtVaGetValues(ggi->iface->textEntry,
		       XtNtextSource, &source, NULL);
	 if (source)
	    XtAddCallback(source, XtNcallback,
			  (XtCallbackProc)echoTextCB, ggi);
      }
   }
   return commentArea;
}

void commentAreaAppend(goGameInterface *ggi, char *str)
{
   Widget source;
   XawTextBlock block;
   XawTextPosition end;

   if (ggi->iface->textArea)
   {
      XtVaSetValues(ggi->iface->textArea,
		    XtNeditType, XawtextAppend,
		    NULL);
      end = XawTextGetInsertionPoint(ggi->iface->textArea);
      block.firstPos = 0;
      block.length = strlen(str);
      block.format = FMT8BIT;
      block.ptr = str;
      XawTextReplace(ggi->iface->textArea, end, end, &block);
      XawTextSetInsertionPoint(ggi->iface->textArea, end+strlen(str));
      XtVaSetValues(ggi->iface->textArea,
		    XtNeditType, XawtextRead,
		    NULL);
   }
}

/* this callback is only installed if socket communication is being used */

void echoTextCB(Widget text, goGameInterface *ggi, XtPointer nothing)
{
   int w = 0, h = 0;
   char *string;

   if (text && ggi)
   {
      XtVaGetValues(text, XtNstring, &string, NULL);

      if (string && string[strlen(string)-1] == '\n')
      {
	 strGetGeometry(string, &w, &h);
	 if (h > 0)
	 {
	    char *at;
	    
	    /* the user has entered a new line of input */
	    if (h > 0 && h <= 2)
	       at = string;
	    else
	    {
	       int i;

	       for (i = strlen(string)-2; i > 0 && string[i] != '\n'; i--);
	       at = &(string[i+1]);
	    }
	    if (at)
	    {
	       /* send the string to display area */
	       /* should we beep on comments? False does not */
	       ggiMessageSend(ggi, False,
			      MESSAGE_COMMENT, 0, 0,
			      goGameGetPlayer(ggi->game), at);
	       /* clear the entered string */
	       XtVaSetValues(text, XtNstring, "", NULL);
	    }
	 }
      }
   }
}


