/* xgoCB.c -- xgo callbacks */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

/* include files: */
#include "xgo.h"		/* all Xm, X11, and std includes */
#include "xIFaceP.h"

#include "goScore.h"

void exit_CB(Widget w, void *stuff, XtPointer x)
{
   goGameInterface *ggi = (goGameInterface *)stuff;
   if (ggi && goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
   {
      ggiMessageSend(ggi, False, MESSAGE_DISCONNECT,
		     0, 0, PLAYER_NONE, NULL);
      goGameInterfaceDisconnect(ggi);
   }
   exit(0);
}

void XtDestroyWidget_CB(Widget w, Widget widget_to_destroy,
				 XtPointer x)
{
   XtDestroyWidget(widget_to_destroy);
}

void XtPopdown_CB(Widget w, Widget widget_to_popdown,
		  XtPointer x)
{
   XtPopdown(widget_to_popdown);
}

void XtFree_CB(Widget w, XtPointer freeMe, XtPointer x)
{
   XtFree(freeMe);
}

void setWidthToParent_CB(Widget w, Widget use, XtPointer x)
{
   Widget parent = NULL;
   Dimension width, border;
   
   if (use) parent = XtParent(use);
   if (parent)
   {
      XtVaGetValues(parent, XtNwidth, &width,
		    XtNborderWidth, &border, NULL);
      if (width)
	 XtVaSetValues(use, XtNwidth, (width-2*border), NULL);
   }
}

/* @@@@ these functions will be broken */

void setPlayerBlackCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   if (ggi)
   {
      goGameSetPlayer(ggi->game, PLAYER_BLACK);
      if (goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
      {
	 setMeLabel(ggi);
      }
      else
      {
	 goGameSetTurn(ggi->game, PLAYER_BLACK);
	 showTurnLabels(ggi);
      }
   }
}

void setPlayerWhiteCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   if (ggi)
   {
      goGameSetPlayer(ggi->game, PLAYER_WHITE);
      if (goGameInterfaceConnectionType(ggi) == CONNECTION_SOCKET)
      {
	 setMeLabel(ggi);
      }
      else
      {
	 goGameSetTurn(ggi->game, PLAYER_WHITE);
	 showTurnLabels(ggi);
      }
   }
}

void setPlayerNoneCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   if (ggi)
      goGameSetPlayer(ggi->game, PLAYER_NONE);
}

void playerPassCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   if (ggi)
   {
      ggiMessageSend(ggi, True, MESSAGE_PASS,
		     0, 0, goGameGetPlayer(ggi->game), NULL);
   }
}

void goGameInterfaceDestroyCB(Widget w, goGameInterface *ggi,
			      XtPointer nothing)
{
   goGameInterfaceDestroy(ggi);
}

void goGameInterfaceClearCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   /* beep for new game == True */
   ggiMessageSend(ggi, True, MESSAGE_NEW_GAME, 0, 0,
		  goGameGetPlayer(ggi->game), NULL);
}

void goGameInterfaceScoreCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   goScore *scoreBlack, *scoreWhite, *scoreComb;

   scoreBlack = goScoreCreate(ggi->game, PLAYER_BLACK);
   scoreWhite = goScoreCreate(ggi->game, PLAYER_WHITE);
   scoreComb = goScoreCreate(ggi->game, PLAYER_NONE);
   if (scoreBlack && scoreWhite && scoreComb)
   {
      goScoreCalculateInfluence(scoreBlack);
      goScoreCalculateInfluence(scoreWhite);
      goScoreCombine(scoreComb, scoreBlack, scoreWhite);
      goScorePrintCombined(scoreComb, 'b', 'w');
      scoreBlack = goScoreDestroy(scoreBlack);
      scoreWhite = goScoreDestroy(scoreWhite);
      scoreComb = goScoreDestroy(scoreComb);
   }
}

void ggiCalcTerritoryCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   ggiMessageSend(ggi, False, MESSAGE_TERRITORY_ON,
		  0, 0, PLAYER_NONE, NULL);
}
