/* xgoFile.c -- saving and loading */

/* mango/tango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include "xgo.h"		/* all Xm, X11, and std includes */
#include "xIFaceP.h"
#include "dialogAPI.h"

#include <errno.h>

typedef struct {
   dialogType *dialog;
   goGameInterface *ggi;
} dialogPacket;

void actuallySaveCB(Widget w, dialogPacket *packet, XtPointer nothing)
{
   if (packet)
   {
      char *str;
      char mesg[512];

      str = dialogGetText(packet->dialog);
      if (!goGameSave(packet->ggi->game, str))
      {
	 sprintf(mesg,"%s: %s", str,
#if defined(STRERROR_MISSING)
		 "File error.  See the terminal."
#else
		 strerror(errno)
#endif
		 );
#if defined(STRERROR_MISSING)
	 perror(str);
#endif
	 dialogErrorPopup(goGameInterfaceGetBoard(packet->ggi),
			  "Save Error", mesg, False);
      }
      XtFree(str);
   }
}

void actuallyLoadCB(Widget w, dialogPacket *packet, XtPointer nothing)
{
   if (packet)
   {
      char *str;
      char mesg[512];

      str = dialogGetText(packet->dialog);
      if (goGameLoad(packet->ggi->game, str))
      {
	 char line[COMMENT_LEN+1];
	 
	 sprintf(line,"Receiving game: %s\n", str);
	 ggiMessageSend(packet->ggi, True, MESSAGE_TRANSFER_HIST,
			0, 0, goGameGetPlayer(packet->ggi->game), line);
	 sprintf(line,"Game loaded: %s\n", str);
	 ggiMessageSend(packet->ggi, True, MESSAGE_TRANSFER_DONE,
			0, 0, goGameGetPlayer(packet->ggi->game), line);
      }
      else
      {
	 sprintf(mesg,"%s: %s", str,
#if defined(STRERROR_MISSING)
		 "File error.  See the terminal."
#else
		 strerror(errno)
#endif
		 );
#if defined(STRERROR_MISSING)
	 perror(str);
#endif
	 dialogErrorPopup(goGameInterfaceGetBoard(packet->ggi),
			  "Load Error", mesg, False);
      }
      XtFree(str);
   }
}

void goGameInterfaceSaveCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   dialogPacket *packet = NEW(dialogPacket);

   if (packet)
   {
      packet->ggi = ggi;
      packet->dialog =
	 dialogCreate(widget_to_shell(w), DIALOG_FILENAME, "Save Game", False);
      if (packet->dialog)
      {
	 dialogSetLabel(packet->dialog, "Enter filename to save to");
	 dialogAddText(packet->dialog);
	 dialogAddButton(packet->dialog, BUTTON_OK, NULL,
			 (XtCallbackProc)actuallySaveCB, packet, True);
	 dialogAddButton(packet->dialog, BUTTON_CANCEL, NULL,
			 NULL, packet, True);
	 dialogPopup(packet->dialog);
      }
   }
}

void goGameInterfaceLoadCB(Widget w, goGameInterface *ggi, XtPointer nothing)
{
   dialogPacket *packet = NEW(dialogPacket);

   if (packet)
   {
      packet->ggi = ggi;
      packet->dialog =
	 dialogCreate(widget_to_shell(w), DIALOG_FILENAME, "Load Game", False);
      if (packet->dialog)
      {
	 dialogSetLabel(packet->dialog, "Enter filename to load");
	 dialogAddText(packet->dialog);
	 dialogAddButton(packet->dialog, BUTTON_OK, NULL,
			 (XtCallbackProc)actuallyLoadCB, packet, True);
	 dialogAddButton(packet->dialog, BUTTON_CANCEL, NULL,
			 NULL, packet, True);
	 dialogPopup(packet->dialog);
      }
   }
}

