/* goMotif.c -- motif version of go functions */

/* xmango -- copyright 1994 Edward Grigassy
 * the code included in this package may be distributed, read, and copied
 * freely BUT NOT SOLD OR INCLUDED IN OTHER SOFTWARE, for commercial or
 * public/free use, without express written consent of the author.
 */

#include "xgo.h"
#include "xIFaceP.h"
#include "dialogAPI.h"

/* ================ pulldown menu convenience routines ================ */

Widget pdm_add_pb(char *item_name, Widget parent, 
		  char *mnemonic, char *label,
		  XtCallbackProc ActCB, XtPointer ActCBData)
		  /* char *accel_text, char *accelerator, */
/* pull-down menu, add push button */
{
    Widget w;
    XmString at, mn, la;
    char *itemName = (item_name? item_name: "MenuButton");

    /* at = XmStringCreate(accel_text,XmSTRING_DEFAULT_CHARSET); */
    la = XmStringCreateLtoR(label,XmSTRING_DEFAULT_CHARSET);

    if ( /* at == NULL || */ la == NULL)
	return NULL;
	
    w = XtVaCreateManagedWidget( itemName,
				xmPushButtonWidgetClass, parent,
				/* XmNacceleratorText, at, */
				/* XmNaccelerator, accelerator, */
				XmNmnemonic, mnemonic[0],
				XmNlabelString, la,
				NULL );
    if (ActCB)
	XtAddCallback(w,XmNactivateCallback,
		      (XtCallbackProc)ActCB,ActCBData);

    /* XmStringFree(at);	*/
    XmStringFree(la);
    return w;
}

Widget pdm_add_sep(char *item_name, Widget parent)
/* pull-down menu add seperator */
{
    Widget w;
    char *itemName = (item_name? item_name: "MenuSeparator");
    
    w = XtVaCreateManagedWidget( itemName, xmSeparatorGadgetClass, parent,
				NULL );
}

Widget mb_create(char *mb_name, Widget parent)
{
    Widget wret;
    char *mbName = (mb_name? mb_name: "MenuBar");

    wret = XtVaCreateManagedWidget( mbName,
				   xmRowColumnWidgetClass, parent,
				   XmNrowColumnType, XmMENU_BAR,
				   NULL );
    if (wret && XmIsForm(parent))
	XtVaSetValues(wret, XmNleftAttachment, XmATTACH_FORM,
		      XmNrightAttachment, XmATTACH_FORM, NULL);
    return wret;
}

Widget mb_add_menu(char *item_name, Widget parent_menu_bar,    
		   char *mnemonic, char *label,
		   Widget *button_widget)
/* menu bar add sub menu */
{
    Widget shell, pdm, button;
    XmString la;
    char *rcName = (item_name? item_name: "MenuPulldown");

    if ( (la = XmStringCreateLtoR(label,XmSTRING_DEFAULT_CHARSET)) == NULL)
    {
	*button_widget = NULL;
	return NULL;
    }

    shell = XtVaCreatePopupShell ("MenuShell",
			      xmMenuShellWidgetClass, parent_menu_bar,
			      XmNwidth, 1,
			      XmNheight, 1,
			      XmNallowShellResize, TRUE,
			      XmNoverrideRedirect, TRUE,
			      NULL );
    pdm = XtVaCreateWidget( rcName,
			   xmRowColumnWidgetClass, shell,
			   XmNrowColumnType, XmMENU_PULLDOWN,
			   NULL );

    button = XtVaCreateManagedWidget( "MenuBarButton",
				xmCascadeButtonWidgetClass, parent_menu_bar,
				XmNmnemonic, mnemonic[0],
				XmNsubMenuId, pdm,
				XmNlabelString, la,
				NULL );
    if (button_widget)
       *button_widget = button;
    XmStringFree(la);
    return (pdm);
}

void XmScrolledTextAppend(Widget text, char *append)
{
   if (text && XmIsText(text))
   {
	XmTextInsert(text, XmTextGetLastPosition(text), append);
	XmTextSetInsertionPosition(text, XmTextGetLastPosition(text));
	XmTextShowPosition(text, XmTextGetLastPosition(text));
   }
}

Widget widget_to_shell(Widget w)
{
   /* don't really care about menu shells... */
   while (w && (!XtIsShell(w) || XmIsMenuShell(w)))
      w = XtParent(w);
      
   return(w);
}

/* -------------------------- building routines -------------------------- */

/* menuBuild -- creates the menu bar and all the sub-menu panes */
/* contained within it */

Widget menuBuild(Widget parent, goGameInterface *ggi)
{
   Widget goMenu = NULL;

   /* expects a form parent */
   if (parent && XmIsForm(parent))
   {
      goMenu =
	 mb_create(NULL, parent);
      if (goMenu)
	 XtVaSetValues(goMenu,
		       XmNtopAttachment, XmATTACH_FORM,
		       NULL);
   }
   if (goMenu)
   {
      Widget gameMenu, stoneMenu;

      gameMenu = mb_add_menu(NULL, goMenu,
			       "G", "Game", NULL);
      if (gameMenu)
      {
	 pdm_add_pb(NULL, gameMenu,
		    "N", "New",
		    goGameInterfaceClearCB, ggi);
	 pdm_add_pb(NULL, gameMenu,
		    "S", "Save",
		    goGameInterfaceSaveCB, ggi);
	 pdm_add_pb(NULL, gameMenu,
		    "L", "Load",
		    goGameInterfaceLoadCB, ggi);
	 pdm_add_pb(NULL, gameMenu,
		    "c", "Score",
		    /* goGameInterfaceScoreCB */
		    ggiCalcTerritoryCB, ggi);
	 pdm_add_sep(NULL, gameMenu);
	 pdm_add_pb(NULL, gameMenu,
		    "E", "Exit",
		    exit_CB, ggi);
      }
      stoneMenu = mb_add_menu(NULL, goMenu,
			      "S", "Stone", NULL);
      if (stoneMenu)
      {
	 pdm_add_pb(NULL, stoneMenu,
		    "U", "Undo Last Move",
		    undoMoveCB, ggi);
	 pdm_add_pb(NULL, stoneMenu,
		    "P", "Pass",
		    playerPassCB, ggi);
	 pdm_add_pb(NULL, stoneMenu,
		    "B", "Set Player: Black",
		    setPlayerBlackCB, ggi);
	 pdm_add_pb(NULL, stoneMenu,
		    "W", "Set Player: White",
		    setPlayerWhiteCB, ggi);
	 ggi->iface->removeButton =
	    pdm_add_pb(NULL, stoneMenu,
		       "R", "Remove Group",
		       removeGroupOfStonesCB, ggi);
	 XtSetSensitive(ggi->iface->removeButton, False);
      }
   }
   return goMenu;
}

Widget statusBuild(Widget parent, goGameInterface *ggi)
{
   Widget form = NULL;

   form = XtVaCreateManagedWidget("status",
				  xmRowColumnWidgetClass, parent,
				  XmNorientation, XmHORIZONTAL,
				  NULL);
   ggi->iface->showMe =
      XtVaCreateManagedWidget("showMe",
			      xmLabelWidgetClass, form,
			      STRING_ARG( XmNlabelString, "(Select Color)" ),
			      NULL);
   ggi->iface->showTurn =
      XtVaCreateManagedWidget("showTurn",
			      xmLabelWidgetClass, form,
			      STRING_ARG( XmNlabelString, "(Select Color)" ),
			      NULL);
   
   return form;
}

void setTurnLabel(goGameInterface *ggi)
{
   if (ggi && ggi->iface->showTurn)
   {
      char info[100];

      sprintf(info, "%s to play.", playerNames[goGameGetTurn(ggi->game)]);
      info[0] = toupper(info[0]);
      XtVaSetValues(ggi->iface->showTurn,
		    STRING_ARG( XmNlabelString, info ),
		    NULL);
   }
}

void setMeLabel(goGameInterface *ggi)
{
   if (ggi && ggi->iface->showMe)
   {
      char info[100];

      sprintf(info, "You are %s.", playerNames[goGameGetPlayer(ggi->game)]);
      XtVaSetValues(ggi->iface->showMe,
		    STRING_ARG( XmNlabelString, info ),
		    NULL);
   }
}

#define DRAWING_AREA_WIDGET_CLASS coreWidgetClass

Widget boardAreaBuild(Widget parent, goGameInterface *ggi)
{
   Widget board = NULL, frame = NULL;

   if (parent)
      frame = XtVaCreateManagedWidget("boardFrame",
				      xmFrameWidgetClass, parent,
				      NULL);
   if (frame)
      board = XtVaCreateManagedWidget("board",
				      DRAWING_AREA_WIDGET_CLASS, frame,
				      NULL);
   if (board)
   {
     int i;
     
     ggi->iface->area = board;
     XtAddEventHandler(board, ExposureMask, False,
		       redrawBoardCB, (XtPointer)ggi);
     XtAddEventHandler(board, StructureNotifyMask, False,
		       redrawBoardCB, (XtPointer)ggi); /* handles resize */
     XtAddEventHandler(board, ButtonPressMask, False,
		       inputBoardCB, (XtPointer)ggi);
   }
   
   return board;
}

#define COMMENT_ROWS	3

Widget commentBuild(Widget parent, goGameInterface *ggi)
{
   Widget commentArea = NULL;
   
   if (parent && ggi)
   {
      Widget scrolledW;
      
      commentArea =
	 XtVaCreateManagedWidget("commentArea",
				 xmFormWidgetClass, parent,
				 NULL);
      ggi->iface->textArea =
	 XmCreateScrolledText(commentArea, "textArea", NULL, 0);
      scrolledW = XtParent(ggi->iface->textArea);
      if (ggi->iface->textArea)
      {
	 XtVaSetValues(ggi->iface->textArea,
		       XmNrows, COMMENT_ROWS,
		       XmNeditMode, XmMULTI_LINE_EDIT,
		       XmNeditable, False,
		       NULL);
	 XtVaSetValues(scrolledW,
		       XmNtopAttachment, XmATTACH_FORM,
		       XmNleftAttachment, XmATTACH_FORM,
		       XmNrightAttachment, XmATTACH_FORM,
		       NULL);
	 XtManageChild(ggi->iface->textArea);
      }

      ggi->iface->textEntry =
	 XtVaCreateManagedWidget("textEntry",
				 xmTextFieldWidgetClass, commentArea,
				 XmNleftAttachment, XmATTACH_FORM,
				 XmNrightAttachment, XmATTACH_FORM,
				 XmNbottomAttachment, XmATTACH_FORM,
				 XmNtopAttachment, XmATTACH_WIDGET,
				 XmNtopWidget, scrolledW,
				 NULL);

      XtAddCallback(ggi->iface->textEntry, XmNactivateCallback,
		    (XtCallbackProc)echoTextCB, ggi);
   }
   return commentArea;
}

void commentAreaAppend(goGameInterface *ggi, char *str)
{
   XmScrolledTextAppend(ggi->iface->textArea, str);
}

/* this callback is only installed if socket communication is being used */

void echoTextCB(Widget text, goGameInterface *ggi, XtPointer nothing)
{
   char *string = NULL;

   if (text && ggi)
   {
      string = XmTextFieldGetString(text);

      if (string)
      {
	 /* send the string to display area */
	 /* @@@@ should we beep on comments? False = no */
	 ggiMessageSend(ggi, False,
			MESSAGE_COMMENT, 0, 0,
			goGameGetPlayer(ggi->game), string);
	 /* clear the entered string */
	 XmTextFieldSetString(text, "");
      }
      XtFree(string);
   }
}

